/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistration } from '@shikijs/types'
import type { ImageStyle, ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import transform, { type StyleTuple } from 'css-to-react-native'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'

const DEBUG = true

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

type NamedStyles<T> = { [P in keyof T]: ViewStyle | TextStyle | ImageStyle }

function createStyleSheet(theme: ThemeRegistration) {
  DEBUG
  && console.log('Creating stylesheet with theme:', {
    colors: theme.colors,
    tokenColors: theme.tokenColors?.length,
  })

  const baseStyles: NamedStyles<{
    scrollView: ViewStyle
    codeContainer: ViewStyle
    lineContainer: ViewStyle
    token: TextStyle
  }> = {
    scrollView: {
      flex: 1,
      minHeight: 20,
    },
    codeContainer: {
      padding: 16,
      minWidth: '100%' as const,
      backgroundColor: theme.colors?.['editor.background'] || '#1e1e1e',
    },
    lineContainer: {
      flexDirection: 'row',
      flexWrap: 'wrap',
    },
    token: {
      fontFamily: monospaceFont,
      fontSize: 14,
      color: theme.colors?.['editor.foreground'] || '#d4d4d4',
    },
  }

  DEBUG && console.log('Created base styles:', baseStyles)
  return StyleSheet.create(baseStyles)
}

const ALLOWED_STYLE_PROPERTIES: Record<string, boolean> = {
  color: true,
  background: true,
  backgroundColor: true,
  fontWeight: true,
  fontStyle: true,
}

function cleanStyle(style: Record<string, any>): TextStyle {
  DEBUG && console.log('Cleaning style:', style)

  const filtered = Object.entries(style)
    .filter(([key]) => ALLOWED_STYLE_PROPERTIES[key])
    .map<StyleTuple>(([key, value]) => [key, value])

  const transformed = transform(filtered)
  DEBUG && console.log('Transformed style:', transformed)
  return transformed
}

interface Props extends SyntaxHighlighterProps {
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const SyntaxHighlighter: React.FC<Props> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  DEBUG
  && console.log('Rendering SyntaxHighlighter with props:', {
    language,
    languageId,
    theme: {
      colors: theme?.colors,
      tokenColors: theme?.tokenColors?.length,
    },
    fontSize,
  })

  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId: languageId || String(language),
    theme,
  })

  DEBUG && console.log('Received tokens:', tokens.length)

  const styles = useMemo(() => createStyleSheet(theme as ThemeRegistration), [theme])

  const renderToken = (token: ThemedToken, rowIndex: number, tokenIndex: number) => {
    DEBUG
    && console.log('Rendering token:', {
      content: token.content,
      color: token.color,
      fontStyle: token.fontStyle,
    })

    const tokenStyle = cleanStyle({
      color: token.color || (styles.token as TextStyle).color,
      fontStyle: token.fontStyle ? 'italic' : 'normal',
      fontWeight: token.fontStyle ? 'bold' : 'normal',
    })

    return (
      <Text key={`${rowIndex}-${tokenIndex}`} style={[styles.token, tokenStyle, textStyle, { fontSize }]}>
        {token.content}
      </Text>
    )
  }

  return (
    <ScrollView
      horizontal
      {...scrollViewProps}
      style={[styles.scrollView, scrollViewProps?.style]}
      contentContainerStyle={[styles.codeContainer, scrollViewProps?.contentContainerStyle, containerStyle]}
    >
      <View onStartShouldSetResponder={() => true}>
        {tokens.map((line, rowIndex) => (
          <View key={rowIndex} style={styles.lineContainer}>
            {line.map((token, tokenIndex) => renderToken(token, rowIndex, tokenIndex))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
